package com.vernaio.example;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import com.vernaio.aivis.engine.v2.base.sdk.EngineLogger;
import com.vernaio.aivis.engine.v2.base.sdk.dto.DtoTabularDataFilter;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.ResponseAnalysis;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.ResponseAnalysisData;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.ResponseAnalysisSetup;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.dto.DtoAnalysisConfig;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.dto.DtoInterest;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.dto.DtoStrategy;
import com.vernaio.aivis.engine.v2.response.analysis.sdk.dto.DtoTargetConfig;

public class Main {
	
	private static final int MIN_LOGGING_LEVEL = 3;
	
	private static final int MAX_THREADS = 0; // max threads 0 means the number of threads will be chosen automatically
	
	private static final String DATA_FILE = "train_ra.csv";
	
	private static final String TARGET_ID = "TARGET";
	
	private static final List<String> STRING_COLUMN_IDS =
		Arrays.asList(
			new String[] {
				"status_of_existing_checking_account",
				"purpose",
				"savings_account",
				"credit_history",
				"marital_status_and_sex",
				"other_debtors",
				"property",
				"other_installment_plans",
				"housing",
				"job",
				"telephone",
				"foreign_worker" }
		);
	
	public static void main(final String[] args) throws Exception {
		
		System.out.println("Setup ...");
		// parsing args as folder config
		final FolderConfig folderConfig = FolderConfig.fromArgs(args);
		// create and register logger
		ResponseAnalysisSetup.registerLogger(new EngineLogger() {
			
			@Override
			public void log(int level, String thread, String module, String message) {
				if (level <= MIN_LOGGING_LEVEL) {
					System.out.println(String.format("\t... %s", message));
				}
			}
		});
		// init thread count
		ResponseAnalysisSetup.initThreadCount(MAX_THREADS);
		System.out.println("\t... done!");
		
		/*
		 * DATA READING
		 */
		
		System.out.println("Loading data ...");
		final ResponseAnalysisData data = Utils.readCsvConvertToAivisFormat(new File(folderConfig.getInputFolder(), DATA_FILE), STRING_COLUMN_IDS);
		
		/*
		 * ANALYSIS
		 */
		
		System.out.println("Creating analysis ...");
		// build analysis config - for more options here, see documentation
		final DtoAnalysisConfig analysisConfig =
			new DtoAnalysisConfig(new DtoTargetConfig(TARGET_ID, DtoInterest.HIGH_KPI)).withDataFilter(
				new DtoTabularDataFilter().withExcludeColumns(new String[] { "telephone" })
			).withStrategy(new DtoStrategy().withMinimalFractionOfRecords(0.05));
		
		// create analysis
		final ResponseAnalysis analysis = ResponseAnalysis.create(data, analysisConfig);
		System.out.println("\t... done!");
		
		// write report file and close
		System.out.println("Retrieving and saving analysis file ...");
		Utils.writeJson(new File(folderConfig.getOutputFolder(), "analysis-report.json"), analysis.getReport());
		System.out.println("\t... done!");
		
		data.destroy();
		System.out.println("\t... all done!");
		return;
		
	}
	
}
